// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-

#include <cstdlib>

#include <fstream>
#include <math.h>
#include <vector>
#include <visp3/core/vpHomogeneousMatrix.h>
#include <visp3/core/vpException.h>

#include <yarp/math/Math.h>
#include <yarp/os/ResourceFinder.h>
#include <yarp/os/RFModule.h>

#include <yarp/sig/Matrix.h>


#include <yarp/os/Network.h>
#include <yarp/os/LogStream.h>
#include <yarp/os/RFModule.h>
#include <yarp/os/Bottle.h>
#include <yarp/sig/Image.h>
#include <yarp/os/PeriodicThread.h>
#include <yarp/os/BufferedPort.h>
#include <yarp/sig/Vector.h>
#include <opencv2/opencv.hpp>

#include <yarp/cv/Cv.h>

// #include <yarp/cv/Cv.h>

// #include <opencv2/core/core.hpp>
// #include <opencv2/core/mat.hpp>
// #include <opencv2/opencv.hpp>

using namespace std;
using namespace yarp::os;
using namespace yarp::sig;

void saveDepthImage(std::string fileName, cv::Mat depthImage){
    std::ofstream fOut(fileName);
    
    for(int i = 0; i < depthImage.rows; i++){
        for(int j = 0; j < depthImage.cols; j++){
            float depth = depthImage.at<float>(i, j);
            fOut << depth << ", ";
        }
        fOut << "\n";
    }
}


class CtrlThread: public PeriodicThread
{
protected:
    BufferedPort<Bottle> inSkeletonPort;
    BufferedPort<ImageOf<PixelRgbFloat>>    imagePort;
    BufferedPort<ImageOf<PixelFloat>>   depthImagePort;
    BufferedPort<Bottle> targetPort;

    Network yarp;


    // camera parameters
    // TODO: load them from some config file
    float focalLengthX = 918.4766845703125;
    float focalLengthY = 916.39385986328125;
    float principalPointX = 639.180908203125;
    float principalPointY = 342.84576416015625;
    int counter_poses_ = 0;
    int number_exp = 1;
    float big_kpt_size = 0.02;
    float small_kpt_size = 0.003;
    

public:
    CtrlThread() : PeriodicThread(1.0) { }

    virtual bool threadInit()
    {
        bool ret=true;
        ret = ret && depthImagePort.open("/skeletonPosition/depthImage:i");

        yarp.connect("/depthCamera/depthImage:o", "/skeletonPosition/depthImage:i");
	    ret = ret && imagePort.open("/skeletonPosition/RgbImage:i");
	    yarp.connect("/depthCamera/rgbImage:o", "/skeletonPosition/RgbImage:i");
    

        return ret;
    }
    /**************************************************/

    virtual void run()
    {   
        // cartesian image is published less frequent than skeleton, so we first read 
        // cart. im. and only after that we check for last published skeleton
        // ImageOf<PixelRgbFloat> *cartesianImage = cartesianImagePort.read(false);

        ImageOf<PixelFloat> *depthImage = depthImagePort.read(false);
        // cout << "Tady" << endl;
        if (depthImage != NULL){
                ImageOf<PixelRgbFloat> *image = imagePort.read(false);
                if (image != NULL){
                /* Save the image. */
                cv::Mat cv_image = yarp::cv::toCvMat(*image);
                cv::imwrite("/home/icub/data_test/image-dataset-" + std::to_string(number_exp) + "_" + std::to_string(counter_poses_) + ".png", cv_image);
                
                cv::Mat cv_depth_image = yarp::cv::toCvMat(*depthImage);
                // cv::imwrite("depth-dataset-" + std::to_string(counter_poses_) + ".png", cv_depth_image);
                saveDepthImage("/home/icub/data_test/depth-dataset-" + std::to_string(number_exp) + "_" + std::to_string(counter_poses_) + ".csv", cv_depth_image);
                cout << "Saved image " << counter_poses_ << endl;
                counter_poses_++;
                }
            }
    }

    virtual void threadRelease()
    {
        depthImagePort.close();
        imagePort.close();
    }
};


class CtrlModule: public RFModule
{
protected:
    CtrlThread thr;

public:
    virtual bool configure(ResourceFinder &rf)
    {
        // retrieve command line options
        double period=rf.check("period",Value(0.02)).asDouble();

        // set the thread period in [s]
        thr.setPeriod(period);

        return thr.start();
    }

    virtual bool close()
    {
        thr.stop();
        return true;
    }

    virtual double getPeriod()
    {
        return 1.0;
    }

    virtual bool updateModule()
    {
        return true;
    }
};



int main(int argc, char *argv[])
{
    Network yarp;
    if (!yarp.checkNetwork())
    {
        yError()<<"YARP doesn't seem to be available";
        return EXIT_FAILURE;
    }

    ResourceFinder rf;
    rf.configure(argc,argv);

    CtrlModule mod;
    return mod.runModule(rf);
}

